/*============================================================================
  File:     GroupingSetsDimensions.sql

  Summary:  This sample shows how many different "rowset types" (dimensions)
			are produced by ROLLUP and CUBE queries with different number of
			columns in the GROUP BY clause.
			
			Also shows how GROUPING SETS can produce only the groupings that
			are needed, a "halfway measure" between ROLLUP and CUBE.

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (RTM)
------------------------------------------------------------------------------
  Written by Bob Beauchemin, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended only as a supplement to SQL Server Jumpstart program.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

use adventureworksdw
go

select count(*) from factinternetsales
go

-- turn on Show Actual Execution Plan 

-- rollup = union all of n groupings
-- cube = union all of 2 to the nth power groupings

-- one group per PK, 158 rows
select productkey, sum(extendedamount)
from factinternetsales
group by productkey

-- subtotals by productkey
-- one dimension, rollup adds total n+1 = 2
select productkey, sum(extendedamount)
from factinternetsales
group by productkey with rollup

-- eqivalent with union all
-- but has to scan table twice
select productkey, sum(extendedamount)
from factinternetsales
group by productkey
union all
select null, sum(extendedamount)
from factinternetsales

-- one dimension, cube == rollup 2 to the first power = 2
select productkey, sum(extendedamount)
from factinternetsales
group by productkey with cube

-- group for each productkey, STkey combo, 1149 rows
select productkey, salesterritorykey, sum(extendedamount)
from factinternetsales
group by productkey, salesterritorykey 

-- one subtotal per product, one grand total, 
-- one group by product, territory
-- two sets of rollup totals n+1 == 2+1 == 3
select productkey, salesterritorykey, sum(extendedamount)
from factinternetsales
group by productkey, salesterritorykey with rollup

-- one set for each salesterritory, one set for each product, 
-- grand total, group by product, sales = 4 groups
select productkey, salesterritorykey, sum(extendedamount)
from factinternetsales
group by productkey, salesterritorykey with cube
order by productkey, salesterritorykey asc

-- 59051 rows - one for each product key, salesterritory, customer
select productkey, salesterritorykey, customerkey, sum(extendedamount)
from factinternetsales
group by productkey, salesterritorykey, customerkey

-- grand total
-- total by productkey
-- total by productkey, salesterritory
-- by productkey, salesterritory, customer
-- 3 sets of total + 1 = 4 groupings
select productkey, salesterritorykey, customerkey, sum(extendedamount)
from factinternetsales
group by productkey, salesterritorykey, customerkey with rollup
order by productkey, salesterritorykey, customerkey

-- grand total
-- by customer
-- by salesterr
-- by salesterr, customer
-- by prod
-- by prod, customer
-- by prod, salesterr
-- by prod, salesterr, customer
-- 8 groupings = 2 to the third power
select productkey, salesterritorykey, customerkey, sum(extendedamount)
from factinternetsales
-- cut down the sheer numbers in the raw data
where salesterritorykey < 5 
  and productkey < 220 
  and customerkey < 11500
group by productkey, salesterritorykey, customerkey with cube
order by productkey, salesterritorykey, customerkey

-- suppose I only want
-- grand total
-- by prod, salesterr, customer
-- by prod, salesterr
select productkey, salesterritorykey, customerkey, sum(extendedamount)
from factinternetsales
-- cut down the sheer numbers in the raw data
where salesterritorykey < 5 
  and productkey < 220 
  and customerkey < 11500
group by grouping sets ((productkey, salesterritorykey, customerkey)
, (productkey, salesterritorykey)
, ())
go

--Can you create an indexed view? 
create view GroupSetsIndexedView
with schemabinding
as
select productkey, salesterritorykey, customerkey, sum(extendedamount)sum_amount
from dbo.factinternetsales
-- cut down the sheer numbers in the raw data
where salesterritorykey < 5 
  and productkey < 220 
  and customerkey < 11500
group by grouping sets ((productkey, salesterritorykey, customerkey)
, (productkey, salesterritorykey)
, ())
go
create unique clustered index IX_GroupSetsIndexedView on GroupSetsIndexedView(productKey,salesterritorykey, customerkey)

--No, because an update would affect multiple rows in the indexed view which is an underlying restrcition on indexed views
