
-- Use Parameters or Literals for Query Inputs:
-- If you use a local variable in a query predicate instead of a parameter or literal, the optimizer is unable to determine the value for the input.

-- estimates 9439.5 (1/3 of the table)
-- chooses merge join
declare @StartOrderDate datetime
set @StartOrderDate = '20040731'
select * from Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderId
AND h.OrderDate >= @StartOrderDate


-- same query, caldinality 40
-- nested loop
SELECT * FROM Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderId
AND h.OrderDate >= '20040731'

-- using sp_executesql appears to use the correct plan
EXEC sp_executesql
	N'SELECT * from Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
      WHERE h.SalesOrderID = d.SalesOrderId
      AND h.OrderDate >= @StartOrderDate',
	N'@StartOrderDate datetime',
	'20040731'

CREATE PROCEDURE GetOrders(@StartOrderDate datetime)
as
--declare @StartOrderDate datetime
--set @StartOrderDate = '20040731'
select * from Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderId
AND h.OrderDate >= @StartOrderDate

execute GetOrders '20040731'

CREATE PROCEDURE GetOrders2(@StartOrderDate datetime)
as
--declare @StartOrderDate datetime
--set @StartOrderDate = '20040731'
select * from Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderId
AND h.OrderDate >= @StartOrderDate
OPTION (OPTIMIZE FOR UNKNOWN)
GO

-- back to merge join
execute GetOrders2 '20040731'

SELECT * FROM Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderId
AND h.OrderDate >= '20040731'
OPTION (OPTIMIZE FOR UNKNOWN)
go

CREATE PROCEDURE GetOrders3(@StartOrderDate datetime)
as
select * from Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderId
AND h.OrderDate >= @StartOrderDate
OPTION (OPTIMIZE FOR (@StartOrderDate = '20040731'))
GO

-- really should be a merge or hash join
-- but optmize for makes it nested loops
execute GetOrders3 '20030701'
GO

DROP PROCEDURE GetRecentSales
GO
CREATE PROCEDURE GetRecentSales (@date datetime)  
AS
BEGIN
	IF @date IS NULL
		SET @date = dateadd("mm",-3,(SELECT MAX(OrderDATE) 
							FROM Sales.SalesOrderHeader))
	SELECT * FROM Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
	WHERE h.SalesOrderID = d.SalesOrderID
	AND h.OrderDate > @date
END

EXECUTE GetRecentSales NULL

-- This SP, if called with NULL, will have the final SELECT statement optimized for @date = NULL. However, at run time, the date is not NULL, but three months before the latest OrderDate.  The actual number of SalesOrderHeader rows that qualify is 5,736. The optimizer chooses a nested loop join for the query when NULL is passed to GetRecentSales, whereas the optimal plan contains a merge join.

execute GetRecentSales NULL WITH RECOMPILE
EXECUTE GetRecentSales '20040430' WITH RECOMPILE

EXECUTE GetRecentSales '20060810' WITH RECOMPILE
go

DROP PROCEDURE GetRecentSalesUnknown
GO
CREATE PROCEDURE GetRecentSalesUnknown (@date datetime)  
AS
BEGIN
	IF @date IS NULL
		SET @date = dateadd("mm",-3,(SELECT MAX(OrderDATE) 
							FROM Sales.SalesOrderHeader))
	SELECT * FROM Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
	WHERE h.SalesOrderID = d.SalesOrderID
	AND h.OrderDate > @date
	OPTION (OPTIMIZE FOR UNKNOWN)
END
go

execute GetRecentSalesUnknown NULL


CREATE PROCEDURE GetRecentSales2 (@date datetime)  AS
BEGIN
	IF @date IS NULL
		SET @date = dateadd("mm",-3,(SELECT MAX(OrderDATE) 		
		 FROM Sales.SalesOrderHeader))
	EXEC GetRecentSalesHelper @date
END
go
DROP PROCEDURE GetRecentSalesHelper
CREATE PROCEDURE GetRecentSalesHelper (@date datetime) -- WITH RECOMPILE 
AS
BEGIN
	SELECT * FROM Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
	WHERE h.SalesOrderID = d.SalesOrderID
	AND h.OrderDate > @date -- @date is unchanged from compile time,
					-- so a good plan is obtained.
END
go

execute GetRecentSales NULL
execute GetRecentSales2 NULL